@@ -555,9 +555,9 @@ def attack_maps_init():
             fr88 = mapx88(from_)
             diff = to88 - fr88
 
-            if diff == 17 or diff == 15:
+            if diff in [17, 15]:
                 m |= attmsk[wP]
-            if diff == -17 or diff == -15:
+            elif diff in [-17, -15]:
                 m |= attmsk[bP]
 
             attmap[to_][from_] = m
@@ -657,7 +657,7 @@ def kapkb_pctoindex(c):
     bk = c.black_piece_squares[0]
     ba = c.black_piece_squares[1]
 
-    if not (chess.A2 <= pawn and pawn < chess.A8):
+    if not (chess.A2 <= pawn < chess.A8):
         return NOINDEX
 
     if (pawn & 7) > 3:
@@ -711,7 +711,7 @@ def kabkp_pctoindex(c):
     bk = c.black_piece_squares[0]
     wb = c.white_piece_squares[2]
 
-    if not (chess.A2 <= pawn and pawn < chess.A8):
+    if not (chess.A2 <= pawn < chess.A8):
         return NOINDEX
 
     if (pawn & 7) > 3:
@@ -1180,7 +1180,7 @@ def kapk_pctoindex(c):
     wk = c.white_piece_squares[0]
     bk = c.black_piece_squares[0]
 
-    if not (chess.A2 <= pawn and pawn < chess.A8):
+    if not (chess.A2 <= pawn < chess.A8):
         return NOINDEX
 
     if (pawn & 7) > 3:
@@ -1234,7 +1234,7 @@ def kakp_pctoindex(c):
     wk = c.white_piece_squares[0]
     bk = c.black_piece_squares[0]
 
-    if not (chess.A2 <= pawn and pawn < chess.A8):
+    if not (chess.A2 <= pawn < chess.A8):
         return NOINDEX
 
     if (pawn & 7) > 3:
@@ -1321,7 +1321,7 @@ def kpk_pctoindex(c):
     wk = c.white_piece_squares[0]
     bk = c.black_piece_squares[0]
 
-    if not (chess.A2 <= pawn and pawn < chess.A8):
+    if not (chess.A2 <= pawn < chess.A8):
         return NOINDEX
 
     if (pawn & 7) > 3:
@@ -1587,7 +1587,7 @@ def adjust_up(dist):
     udist = dist
     sw = udist & INFOMASK
 
-    if (sw == iWMATE) or (sw == iWMATEt) or (sw == iBMATE) or (sw == iBMATEt):
+    if sw in [iWMATE, iWMATEt, iBMATE, iBMATEt]:
         udist += (1 << PLYSHIFT)
 
     return udist
@@ -1627,7 +1627,7 @@ def unpackdist(d):
 def dtm_unpack(stm, packed):
     p = packed
 
-    if iDRAW == p or iFORBID == p:
+    if p in [iDRAW, iFORBID]:
         return p
 
     info = p & 3
@@ -1874,7 +1874,7 @@ class PythonTablebases(object):
         return self._open_tablebase(req)
 
     def _open_tablebase(self, req):
-        stream = self.streams.get(req.egkey, None)
+        stream = self.streams.get(req.egkey)
 
         if stream is None:
             path = self.available_tables[req.egkey]
@@ -2079,7 +2079,10 @@ class PythonTablebases(object):
 
 
 class NativeTablebases(object):
-    """Provides access to Gaviota tablebases via the shared library libgtb."""
+    """
+    Provides access to Gaviota tablebases via the shared library libgtb.
+    Has the same interface as :class:`~chess.gaviota.PythonTablebases`.
+    """
 
     def __init__(self, directory, libgtb):
         self.libgtb = libgtb
@@ -2234,6 +2237,15 @@ class NativeTablebases(object):
 
 
 def open_tablebases_native(directory, libgtb=None, LibraryLoader=ctypes.cdll):
+    """
+    Opens a collection of tablebases for probing using libgtb.
+
+    In most cases :func:`~chess.gaviota.open_tablebases()` should be used.
+    Use this function only if you do not want to downgrade to pure Python
+    tablebase probing.
+
+    Raises :exc:`RuntimeError` or :exc:`OSError` when libgtb can not be used.
+    """
     libgtb = libgtb or ctypes.util.find_library("gtb") or "libgtb.so.1.0.1"
     return NativeTablebases(directory, LibraryLoader.LoadLibrary(libgtb))
 
