@@ -19,12 +19,17 @@
 import chess
 import itertools
 import re
+import logging
 
 try:
     import backport_collections as collections
 except ImportError:
     import collections
 
+
+LOGGER = logging.getLogger(__name__)
+
+
 NAG_NULL = 0
 
 NAG_GOOD_MOVE = 1
@@ -240,73 +245,69 @@ class GameNode(object):
         self.promote_to_main(move)
         return node
 
-    def export(self, exporter, comments=True, variations=True, _board=None, _after_variation=False):
-        if _board is None:
-            _board = self.board()
+    def accept(self, visitor, _board=None):
+        """
+        Traverse game nodes in PGN order using the given *visitor*. Returns
+        the visitor result.
+        """
+        board = self.board() if _board is None else _board
 
         # The mainline move goes first.
         if self.variations:
             main_variation = self.variations[0]
+            visitor.visit_move(board, main_variation.move)
 
-            # Append fullmove number.
-            exporter.put_fullmove_number(_board.turn, _board.fullmove_number, _after_variation)
+            # Visit NAGs.
+            for nag in sorted(main_variation.nags):
+                visitor.visit_nag(nag)
 
-            # Append SAN.
-            exporter.put_move(_board, main_variation.move)
+            # Visit the comment.
+            if main_variation.comment:
+                visitor.visit_comment(main_variation.comment)
 
-            if comments:
-                # Append NAGs.
-                exporter.put_nags(main_variation.nags)
+        # Then visit sidelines.
+        for variation in itertools.islice(self.variations, 1, None):
+            # Start variation.
+            visitor.begin_variation()
 
-                # Append the comment.
-                if main_variation.comment:
-                    exporter.put_comment(main_variation.comment)
+            # Append starting comment.
+            if variation.starting_comment:
+                visitor.visit_comment(variation.starting_comment)
 
-        # Then export sidelines.
-        if variations:
-            for variation in itertools.islice(self.variations, 1, None):
-                # Start variation.
-                exporter.start_variation()
+            # Visit move.
+            visitor.visit_move(board, variation.move)
 
-                # Append starting comment.
-                if comments and variation.starting_comment:
-                    exporter.put_starting_comment(variation.starting_comment)
+            # Visit NAGs.
+            for nag in sorted(variation.nags):
+                visitor.visit_nag(nag)
 
-                # Append fullmove number.
-                exporter.put_fullmove_number(_board.turn, _board.fullmove_number, True)
+            # Visit comment.
+            if variation.comment:
+                visitor.visit_comment(variation.comment)
 
-                # Append SAN.
-                exporter.put_move(_board, variation.move)
+            # Recursively append the next moves.
+            board.push(variation.move)
+            variation.accept(visitor, _board=board)
+            board.pop()
 
-                if comments:
-                    # Append NAGs.
-                    exporter.put_nags(variation.nags)
-
-                    # Append the comment.
-                    if variation.comment:
-                        exporter.put_comment(variation.comment)
-
-                # Recursively append the next moves.
-                _board.push(variation.move)
-                variation.export(exporter, comments, variations, _board, _after_variation=False)
-                _board.pop()
-
-                # End variation.
-                exporter.end_variation()
+            # End variation.
+            visitor.end_variation()
 
         # The mainline is continued last.
         if self.variations:
             main_variation = self.variations[0]
 
             # Recursively append the next moves.
-            _board.push(main_variation.move)
-            main_variation.export(exporter, comments, variations, _board, _after_variation=variations and len(self.variations) > 1)
-            _board.pop()
+            board.push(main_variation.move)
+            main_variation.accept(visitor, _board=board)
+            board.pop()
+
+        # Get the result if not called recursively.
+        if _board is None:
+            return visitor.result()
 
     def __str__(self):
-        exporter = StringExporter(columns=None)
-        self.export(exporter)
-        return exporter.__str__()
+        return self.accept(StringExporter(columns=None))
 
 
 class Game(GameNode):
@@ -358,7 +359,7 @@ class Game(GameNode):
         starting position.
         """
         if "FEN" in self.headers and "SetUp" in self.headers and self.headers["SetUp"] == "1":
-            chess960 = self.headers.get("Variant", None) == "Chess960"
+            chess960 = self.headers.get("Variant") == "Chess960"
             board = chess.Board(self.headers["FEN"], chess960=chess960)
             board.chess960 = board.chess960 or board.has_chess960_castling_rights()
             return board
@@ -389,46 +390,186 @@ class Game(GameNode):
         else:
             self.headers.pop("Variant", None)
 
-    def export(self, exporter, headers=True, comments=True, variations=True):
-        exporter.start_game()
+    def accept(self, visitor):
+        """
+        Traverses the game in PGN order using the given *visitor*. Returns
+        the visitor result.
+        """
+        visitor.begin_game()
 
-        if headers:
-            exporter.start_headers()
-            for tagname, tagvalue in self.headers.items():
-                exporter.put_header(tagname, tagvalue)
-            exporter.end_headers()
+        visitor.begin_headers()
+        for tagname, tagvalue in self.headers.items():
+            visitor.visit_header(tagname, tagvalue)
+        visitor.end_headers()
 
-        if comments and self.comment:
-            exporter.put_starting_comment(self.comment)
+        if self.comment:
+            visitor.visit_comment(self.comment)
 
-        super(Game, self).export(exporter, comments=comments, variations=variations, _after_variation=True)
+        super(Game, self).accept(visitor, _board=self.board())
 
-        exporter.put_result(self.headers["Result"])
-        exporter.end_game()
+        visitor.visit_result(self.headers["Result"])
+        visitor.end_game()
+        return visitor.result()
 
 
-class StringExporter(object):
+class BaseVisitor(object):
+    """
+    Base class for visitors.
+
+    Use with :func:`chess.pgn.Game.accept()` or
+    :func:`chess.pgn.GameNode.accept()`.
+
+    Methods are called in PGN order.
+    """
+
+    def begin_game(self):
+        """Called at the start of a game."""
+        pass
+
+    def end_game(self):
+        """Called at the end of a game."""
+        pass
+
+    def begin_headers(self):
+        """Called at the start of the game headers."""
+        pass
+
+    def visit_header(self, tagname, tagvalue):
+        """Called for each game header."""
+        pass
+
+    def end_headers(self):
+        """Called at the end of the game headers."""
+        pass
+
+    def begin_variation(self):
+        """
+        Called at the start of a new variation. It is not called for the
+        mainline of the game.
+        """
+        pass
+
+    def end_variation(self):
+        """Concludes a variation."""
+        pass
+
+    def visit_comment(self, comment):
+        """Called for each comment."""
+        pass
+
+    def visit_nag(self, nag):
+        """Called for each NAG."""
+        pass
+
+    def visit_move(self, board, move):
+        """
+        Called for each move.
+
+        *board* is the board state before the move. The board state must be
+        restored before the traversal continues.
+        """
+        pass
+
+    def visit_result(self, result):
+        """Called at the end of the game with the *Result*-header."""
+        pass
+
+    def handle_error(self, error):
+        """Called for errors encountered. Defaults to raising an exception."""
+        raise error
+
+    def result(self):
+        """Called to get the result of the visitor. Defaults to ``None``."""
+        return None
+
+
+class GameModelCreator(BaseVisitor):
+    """
+    Creates a game model. Default visitor for :func:`~chess.pgn.read_game()`.
+    """
+
+    def __init__(self):
+        self.game = Game()
+        self.found_game = False
+
+        self.variation_stack = collections.deque([self.game])
+        self.starting_comment = ""
+        self.in_variation = False
+
+    def begin_game(self):
+        self.found_game = True
+
+    def visit_header(self, tagname, tagvalue):
+        self.game.headers[tagname] = tagvalue
+
+    def visit_nag(self, nag):
+        self.variation_stack[-1].nags.add(nag)
+
+    def begin_variation(self):
+        self.variation_stack.append(self.variation_stack[-1].parent)
+        self.in_variation = False
+
+    def end_variation(self):
+        self.variation_stack.pop()
+
+    def visit_result(self, result):
+        if self.game.headers.get("Result", "*") == "*":
+            self.game.headers["Result"] = result
+
+    def visit_comment(self, comment):
+        if self.in_variation or (not self.variation_stack[-1].parent and self.variation_stack[-1].is_end()):
+            # Add as a comment for the current node if in the middle of
+            # a variation. Add as a comment for the game, if the comment
+            # starts before any move.
+            new_comment = [self.variation_stack[-1].comment, comment]
+            self.variation_stack[-1].comment = "\n".join(new_comment).strip()
+        else:
+            # Otherwise it is a starting comment.
+            new_comment = [self.starting_comment, comment]
+            self.starting_comment = "\n".join(new_comment).strip()
+
+    def visit_move(self, board, move):
+        self.variation_stack[-1] = self.variation_stack[-1].add_variation(move)
+        self.variation_stack[-1].starting_comment = self.starting_comment
+        self.starting_comment = ""
+        self.in_variation = True
+
+    def handle_error(self, error):
+        LOGGER.exception("error during pgn parsing")
+
+    def result(self):
+        """
+        Returns a :class:`~chess.pgn.Game()` or ``None`` if no game was
+        encountered.
+        """
+        return self.game if self.found_game else None
+
+
+class StringExporter(BaseVisitor):
     """
     Allows exporting a game as a string.
 
-    :func:`chess.pgn.Game.export()` also provides options to include or exclude
-    headers, variations or comments. By default everything is included.
-
     >>> exporter = chess.pgn.StringExporter()
-    >>> game.export(exporter, headers=True, variations=True, comments=True)
-    >>> pgn_string = str(exporter)
+    >>> pgn_string = game.accept(exporter, headers=True, variations=True, comments=True)
 
-    Only `columns` characters are written per line. If `columns` is ``None``
+    Only *columns* characters are written per line. If *columns* is ``None``
     then the entire movetext will be on a single line. This does not affect
     header tags and comments.
 
     There will be no newlines at the end of the string.
     """
 
-    def __init__(self, columns=80):
-        self.lines = []
+    def __init__(self, columns=80, headers=True, comments=True, variations=True):
         self.columns = columns
+        self.headers = headers
+        self.comments = comments
+        self.variations = variations
+
+        self.force_movenumber = True
+
+        self.lines = []
         self.current_line = ""
+        self.variation_depth = 0
 
     def flush_current_line(self):
         if self.current_line:
@@ -444,58 +585,68 @@ class StringExporter(object):
         self.flush_current_line()
         self.lines.append(line.rstrip())
 
-    def start_game(self):
-        pass
+    def begin_game(self):
+        self.after_variation = True
 
     def end_game(self):
         self.write_line()
 
-    def start_headers(self):
-        pass
-
-    def put_header(self, tagname, tagvalue):
-        self.write_line("[{0} \"{1}\"]".format(tagname, tagvalue))
+    def visit_header(self, tagname, tagvalue):
+        if self.headers:
+            self.write_line("[{0} \"{1}\"]".format(tagname, tagvalue))
 
     def end_headers(self):
-        self.write_line()
+        if self.headers:
+            self.write_line()
 
-    def start_variation(self):
-        self.write_token("( ")
+    def begin_variation(self):
+        self.variation_depth += 1
+
+        if self.variations:
+            self.write_token("( ")
+            self.force_movenumber = True
 
     def end_variation(self):
-        self.write_token(") ")
+        self.variation_depth -= 1
 
-    def put_starting_comment(self, comment):
-        self.put_comment(comment)
+        if self.variations:
+            self.write_token(") ")
+            self.force_movenumber = True
 
-    def put_comment(self, comment):
-        self.write_token("{ " + comment.replace("}", "").strip() + " } ")
+    def visit_comment(self, comment):
+        if self.comments and (self.variations or not self.variation_depth):
+            self.write_token("{ " + comment.replace("}", "").strip() + " } ")
+            self.force_movenumber = True
 
-    def put_nags(self, nags):
-        for nag in sorted(nags):
-            self.put_nag(nag)
+    def visit_nag(self, nag):
+        if self.comments and (self.variations or not self.variation_depth):
+            self.write_token("$" + str(nag) + " ")
 
-    def put_nag(self, nag):
-        self.write_token("$" + str(nag) + " ")
+    def visit_move(self, board, move):
+        if self.variations or not self.variation_depth:
+            # Write the move number.
+            if board.turn == chess.WHITE:
+                self.write_token(str(board.fullmove_number) + ". ")
+            elif self.force_movenumber:
+                self.write_token(str(board.fullmove_number) + "... ")
 
-    def put_fullmove_number(self, turn, fullmove_number, variation_start):
-        if turn == chess.WHITE:
-            self.write_token(str(fullmove_number) + ". ")
-        elif variation_start:
-            self.write_token(str(fullmove_number) + "... ")
+            # Write the SAN.
+            self.write_token(board.san(move) + " ")
 
-    def put_move(self, board, move):
-        self.write_token(board.san(move) + " ")
+            self.force_movenumber = False
 
-    def put_result(self, result):
+    def visit_result(self, result):
         self.write_token(result + " ")
 
-    def __str__(self):
+    def result(self):
         if self.current_line:
             return "\n".join(itertools.chain(self.lines, [self.current_line.rstrip()])).rstrip()
         else:
             return "\n".join(self.lines).rstrip()
 
+    def __str__(self):
+        return self.result()
+
 
 class FileExporter(StringExporter):
     """
@@ -505,13 +656,13 @@ class FileExporter(StringExporter):
     There will always be a blank line after each game. Handling encodings is up
     to the caller.
 
-    >>> new_pgn = open("new.pgn", "w")
+    >>> new_pgn = open("new.pgn", "w", encoding="utf-8")
     >>> exporter = chess.pgn.FileExporter(new_pgn)
-    >>> game.export(exporter)
+    >>> game.accept(exporter)
     """
 
-    def __init__(self, handle, columns=80):
-        super(FileExporter, self).__init__(columns=columns)
+    def __init__(self, handle, columns=80, headers=True, comments=True, variations=True):
+        super(FileExporter, self).__init__(columns=columns, headers=headers, comments=comments, variations=variations)
         self.handle = handle
 
     def flush_current_line(self):
@@ -525,20 +676,20 @@ class FileExporter(StringExporter):
         self.handle.write(line.rstrip())
         self.handle.write("\n")
 
+    def result(self):
+        return None
 
-def _raise(error):
-    raise error
+    def __repr__(self):
+        return "<FileExporter at {0}>".format(hex(id(self)))
+
+    def __str__(self):
+        return self.__repr__()
 
 
-def read_game(handle, error_handler=_raise):
+def read_game(handle, Visitor=GameModelCreator):
     """
     Reads a game from a file opened in text mode.
 
-    By using text mode the parser does not need to handle encodings. It is the
-    callers responsibility to open the file with the correct encoding.
-    According to the specification PGN files should be ASCII. Also UTF-8 is
-    common. So this is usually not a problem.
-
     >>> pgn = open("data/pgn/kasparov-deep-blue-1997.pgn")
     >>> first_game = chess.pgn.read_game(pgn)
     >>> second_game = chess.pgn.read_game(pgn)
@@ -546,6 +697,14 @@ def read_game(handle, error_handler=_raise):
     >>> first_game.headers["Event"]
     'IBM Man-Machine, New York USA'
 
+    By using text mode the parser does not need to handle encodings. It is the
+    callers responsibility to open the file with the correct encoding.
+    PGN files are ASCII or UTF-8 most of the time. So the following should
+    cover most relevant cases (ASCII, UTF-8 without BOM, UTF-8 with BOM,
+    UTF-8 with encoding errors).
+
+    >>> pgn = open("data/pgn/kasparov-deep-blue-1997.pgn", encoding="utf-8-sig", errors="surrogateescape")
+
     Use `StringIO` to parse games from a string.
 
     >>> pgn_string = "1. e4 e5 2. Nf3 *"
@@ -566,16 +725,13 @@ def read_game(handle, error_handler=_raise):
     fine.
 
     The parser is relatively forgiving when it comes to errors. It skips over
-    tokens it can not parse. However it is difficult to handle illegal or
-    ambiguous moves. If such a move is encountered the default behaviour is to
-    stop right in the middle of the game and raise :exc:`ValueError`. If you
-    pass ``None`` for *error_handler* all errors are silently ignored, instead.
-    If you pass a function this function will be called with the error as an
-    argument.
+    tokens it can not parse. Any exceptions are logged.
 
     Returns the parsed game or ``None`` if the EOF is reached.
     """
-    game = Game()
+    visitor = Visitor()
+
+    dummy_game = Game()
     found_game = False
     found_content = False
 
@@ -588,34 +744,42 @@ def read_game(handle, error_handler=_raise):
             line = handle.readline()
             continue
 
+        if not found_game:
+            visitor.begin_game()
+            visitor.begin_headers()
+
         found_game = True
 
         # Read header tags.
         tag_match = TAG_REGEX.match(line)
         if tag_match:
-            game.headers[tag_match.group(1)] = tag_match.group(2)
+            dummy_game.headers[tag_match.group(1)] = tag_match.group(2)
+            visitor.visit_header(tag_match.group(1), tag_match.group(2))
         else:
             break
 
         line = handle.readline()
 
+    if found_game:
+        visitor.end_headers()
+
     # Get the next non-empty line.
     while not line.strip() and line:
         line = handle.readline()
 
     # Movetext parser state.
-    starting_comment = ""
-    variation_stack = collections.deque([game])
-    board_stack = collections.deque([game.board()])
-    in_variation = False
+    board_stack = collections.deque([dummy_game.board()])
 
     # Parse movetext.
     while line:
         read_next_line = True
 
         # An empty line is the end of a game.
-        if not line.strip() and found_game and found_content:
-            return game
+        if not line.strip() and found_content:
+            if found_game:
+                visitor.end_game()
+
+            return visitor.result()
 
         for match in MOVETEXT_REGEX.finditer(line):
             token = match.group(0)
@@ -625,7 +789,9 @@ def read_game(handle, error_handler=_raise):
                 line = handle.readline()
                 continue
 
-            found_game = True
+            if not found_game:
+                found_game = True
+                visitor.begin_game()
 
             if token.startswith("{"):
                 # Consume until the end of the comment.
@@ -641,18 +807,7 @@ def read_game(handle, error_handler=_raise):
                 else:
                     line = ""
 
-                if in_variation or (not variation_stack[-1].parent and variation_stack[-1].is_end()):
-                    # Add as a comment for the current node if in the middle of
-                    # a variation. Add as a comment for the game, if the
-                    # comment starts before any move.
-                    if variation_stack[-1].comment:
-                        comment_lines.insert(0, variation_stack[-1].comment)
-                    variation_stack[-1].comment = "\n".join(comment_lines).strip()
-                else:
-                    # Otherwise it is a starting comment.
-                    if starting_comment:
-                        comment_lines.insert(0, starting_comment)
-                    starting_comment = "\n".join(comment_lines).strip()
+                visitor.visit_comment("\n".join(comment_lines).strip())
 
                 # Continue with the current or the next line.
                 if line:
@@ -660,42 +815,41 @@ def read_game(handle, error_handler=_raise):
                 break
             elif token.startswith("$"):
                 # Found a NAG.
-                variation_stack[-1].nags.add(int(token[1:]))
+                try:
+                    nag = int(token[1:])
+                except ValueError as error:
+                    visitor.handle_error(error)
+                else:
+                    visitor.visit_nag(nag)
             elif token == "?":
-                variation_stack[-1].nags.add(NAG_MISTAKE)
+                visitor.visit_nag(NAG_MISTAKE)
             elif token == "??":
-                variation_stack[-1].nags.add(NAG_BLUNDER)
+                visitor.visit_nag(NAG_BLUNDER)
             elif token == "!":
-                variation_stack[-1].nags.add(NAG_GOOD_MOVE)
+                visitor.visit_nag(NAG_GOOD_MOVE)
             elif token == "!!":
-                variation_stack[-1].nags.add(NAG_BRILLIANT_MOVE)
+                visitor.visit_nag(NAG_BRILLIANT_MOVE)
             elif token == "!?":
-                variation_stack[-1].nags.add(NAG_SPECULATIVE_MOVE)
+                visitor.visit_nag(NAG_SPECULATIVE_MOVE)
             elif token == "?!":
-                variation_stack[-1].nags.add(NAG_DUBIOUS_MOVE)
+                visitor.visit_nag(NAG_DUBIOUS_MOVE)
             elif token == "(":
-                # Found a start variation token.
-                if variation_stack[-1].parent:
-                    variation_stack.append(variation_stack[-1].parent)
+                if board_stack[-1].move_stack:
+                    visitor.begin_variation()
 
                     board = board_stack[-1].copy()
                     board.pop()
                     board_stack.append(board)
-
-                    in_variation = False
             elif token == ")":
                 # Found a close variation token. Always leave at least the
                 # root node on the stack.
-                if len(variation_stack) > 1:
-                    variation_stack.pop()
+                if len(board_stack) > 1:
+                    visitor.end_variation()
                     board_stack.pop()
-            elif token in ["1-0", "0-1", "1/2-1/2", "*"] and len(variation_stack) == 1:
+            elif token in ["1-0", "0-1", "1/2-1/2", "*"] and len(board_stack) == 1:
                 # Found a result token.
                 found_content = True
-
-                # Set result header if not present, yet.
-                if game.headers.get("Result", "*") == "*":
-                    game.headers["Result"] = token
+                visitor.visit_result(token)
             else:
                 # Found a SAN token.
                 found_content = True
@@ -709,21 +863,19 @@ def read_game(handle, error_handler=_raise):
                 # Parse the SAN.
                 try:
                     move = board_stack[-1].parse_san(token)
-                    in_variation = True
-                    variation_stack[-1] = variation_stack[-1].add_variation(move)
-                    variation_stack[-1].starting_comment = starting_comment
-                    board_stack[-1].push(move)
-                    starting_comment = ""
                 except ValueError as error:
-                    game.errors.append(error)
-                    if error_handler:
-                        error_handler(error)
+                    visitor.handle_error(error)
+                else:
+                    visitor.visit_move(board_stack[-1], move)
+                    board_stack[-1].push(move)
 
         if read_next_line:
             line = handle.readline()
 
     if found_game:
-        return game
+        visitor.end_game()
+
+    return visitor.result()
 
 
 def scan_headers(handle):
